--[==[
Copyright ©2019 Samuel Thomas Pain

The contents of this addon, excluding third-party resources, are
copyrighted to their authors with all rights reserved.

This addon is free to use and the authors hereby grants you the following rights:

1. 	You may make modifications to this addon for private use only, you
    	may not publicize any portion of this addon.

2. 	Do not modify the name of this addon, including the addon folders.

3. 	This copyright notice shall be included in all copies or substantial
    	portions of the Software.

All rights not explicitly addressed in this license are reserved by
the copyright holders.
]==]--


Wardrobe = { UI = {}, DB = {}, Vars = { CurrentlyEquippedOutfit = nil, SelectedOutfit = nil, Debug = false, TooltipLineAdded = false } }
Wardrobe.EventFrame = CreateFrame("FRAME", "Wardrobe_EventFrame", UIParent)
Wardrobe.EventFrame:RegisterEvent("ADDON_LOADED")
Wardrobe.EventFrame:RegisterEvent("BANKFRAME_OPENED")
Wardrobe.EventFrame:RegisterEvent("BANKFRAME_CLOSED")

function Wardrobe.GetArgs(...)
    for i=1, select("#", ...) do
        arg = select(i, ...)
        print(i.." "..tostring(arg))
    end
end

function Wardrobe.PrintMessage(msg)
    DEFAULT_CHAT_FRAME:AddMessage("|cffFF7D0AWardrobe: |r"..msg)
end

function Wardrobe.DebugMessage(msg)
    if Wardrobe.Vars.Debug == true then
        DEFAULT_CHAT_FRAME:AddMessage("|cffC41F3BWardrobe DEBUG: |r"..msg)
    end
end

Wardrobe.DB.InvSlots = {
    [0] = 'AMMOSLOT',
    [1] = 'HEADSLOT',
    [2] = 'NECKSLOT',
    [3] = 'SHOULDERSLOT',
    [4] = 'SHIRTSLOT',
    [5] = 'CHESTSLOT',
    [6] = 'WAISTSLOT',
    [7] = 'LEGSSLOT',
    [8] = 'FEETSLOT',
    [9] = 'WRISTSLOT',
    [10] = 'HANDSSLOT',
    [11] = 'FINGER0SLOT',
    [12] = 'FINGER1SLOT',
    [13] = 'TRINKET0SLOT',
    [14] = 'TRINKET1SLOT',
    [15] = 'BACKSLOT',
    [16] = 'MAINHANDSLOT',
    [17] = 'SECONDARYHANDSLOT',
    [18] = 'RANGEDSLOT',
    [19] = 'TABARDSLOT',
}

SLASH_WARDROBE1 = '/wardrobe'
SlashCmdList['WARDROBE'] = function(msg)

    if msg == 'help' then
        print('no help!')
        Wardrobe.FixDb()

    elseif msg == 'debug-ON' then
        Wardrobe.Vars.Debug = true
        DEFAULT_CHAT_FRAME:AddMessage("|cffC41F3BWardrobe DEBUG: |r"..'debug is on!')

    elseif msg == 'debug-OFF' then
        Wardrobe.Vars.Debug = false
        DEFAULT_CHAT_FRAME:AddMessage("|cffC41F3BWardrobe DEBUG: |r"..'debug is off!')
        

    elseif string.find(msg, 'equipset-') then
        if WardrobeDb[UnitGUID('player')] ~= nil then
            for k, outfit in ipairs(WardrobeDb[UnitGUID('player')].Outfits) do
                if outfit.Name == string.sub(msg, 10) then
                    Wardrobe.Vars.SelectedOutfit = outfit.Name
                    UIDropDownMenu_SetText(Wardrobe.UI.OutfitFrame.OutfitDropDown, outfit.Name)
                    --Wardrobe.EquipCurrentSet()
                    Wardrobe.IterateSet(outfit.Name)
                end
            end
        end


    end

end

StaticPopupDialogs["Wardrobe_NewOutfit"] = {
	text = "New outfit name", button1 = "Ok", button2 = "Cancel",
    hasEditBox = true, EditBoxOnTextChanged = function(self) if self:GetText() ~= '' then self:GetParent().button1:Enable() else self:GetParent().button1:Disable() end end,
	OnShow = function(self) self.editBox:SetText('') self.button1:Disable() end,
	OnAccept = function(self) Wardrobe.CreateNewOutfit(self.editBox:GetText()) end,
	timeout = 0, whileDead = true, hideOnEscape = true,	preferredIndex = 3,
}

--tooltip info, uses item ID which is still an issue where a player may have 2 items of the same name but with different random enchants (of the owl, of the boar) i assume the item keeps the same id but would have a different link?
function Wardrobe.OnTooltipSetItem(tooltip, ...)
    local ItemName, ItemLink = GameTooltip:GetItem()
    local setsString = nil
    if WardrobeDb[UnitGUID('player')] ~= nil then
        for k, outfit in ipairs(WardrobeDb[UnitGUID('player')].Outfits) do
            for slot, itemID in pairs(outfit) do
                local added = false
                if added == true then break end
                if slot ~= 'Name' then
                    local id = select(1, GetItemInfoInstant(ItemLink))
                    if id == itemID then
                        if setsString == nil then
                            setsString = outfit.Name
                            added = true
                        else
                            setsString = tostring(setsString..', '..outfit.Name)
                            added = true
                        end
                    end
                end
            end
        end
    end
    if setsString ~= nil then
        if not Wardrobe.Vars.TooltipLineAdded then
            tooltip:AddLine(' ') --create a line break
            tooltip:AddLine("Sets", 1.00, 0.49, 0.04)
            tooltip:AddLine(setsString, 1, 1, 1)
            Wardrobe.Vars.TooltipLineAdded = true
        end
    end
end

function Wardrobe.OnTooltipCleared(tooltip, ...)
    Wardrobe.Vars.TooltipLineAdded = false
end

GameTooltip:HookScript("OnTooltipSetItem", Wardrobe.OnTooltipSetItem)
GameTooltip:HookScript("OnTooltipCleared", Wardrobe.OnTooltipCleared)

Wardrobe.UI.OpenWardrobeButton = CreateFrame("BUTTON", "Wardrobe_OpenButton", PaperDollFrame) --, "UIPanelSquareButton") -- "UIPanelButtonTemplate")
Wardrobe.UI.OpenWardrobeButton:SetPoint("TOPRIGHT", -40, -40)
Wardrobe.UI.OpenWardrobeButton:SetSize(35, 30)
Wardrobe.UI.OpenWardrobeButton.Texture = Wardrobe.UI.OpenWardrobeButton:CreateTexture("$parent_Texture", "OVERLAY")
Wardrobe.UI.OpenWardrobeButton.Texture:SetAllPoints(Wardrobe.UI.OpenWardrobeButton)
Wardrobe.UI.OpenWardrobeButton.Texture:SetTexture("interface/paperdollinfoframe/ui-gearmanager-button")
Wardrobe.UI.OpenWardrobeButton:SetScript("OnEnter", function(self) self:SetAlpha(0.85) end)
Wardrobe.UI.OpenWardrobeButton:SetScript("OnLeave", function(self) self:SetAlpha(1.0) end)
Wardrobe.UI.OpenWardrobeButton:SetScript("OnMouseDown", function(self) self.Texture:SetTexture("interface/paperdollinfoframe/ui-gearmanager-button-pushed") end)
Wardrobe.UI.OpenWardrobeButton:SetScript("OnMouseUp", function(self) self.Texture:SetTexture("interface/paperdollinfoframe/ui-gearmanager-button") end)
Wardrobe.UI.OpenWardrobeButton:SetScript("OnClick", function() if Wardrobe.UI.OutfitFrame:IsVisible() then Wardrobe.UI.OutfitFrame:Hide() else Wardrobe.UI.OutfitFrame:Show() end end)

Wardrobe.UI.OutfitFrame = CreateFrame("FRAME", "Wardrobe_Outfitframe", PaperDollFrame)
Wardrobe.UI.OutfitFrame:SetBackdrop({ bgFile = "Interface/Tooltips/UI-Tooltip-Background", edgeFile = "interface/dialogframe/ui-dialogbox-border", tile = true, tileSize = 16, edgeSize = 20, insets = { left = 4, right = 4, top = 4, bottom = 4 }});
Wardrobe.UI.OutfitFrame:SetBackdropColor(0, 0, 0, 0.9)
Wardrobe.UI.OutfitFrame:SetSize(240, 80)
Wardrobe.UI.OutfitFrame:SetPoint("TOPLEFT", Wardrobe.UI.OpenWardrobeButton, "TOPLEFT", 45, 0)
Wardrobe.UI.OutfitFrame:Hide()

Wardrobe.UI.OutfitFrame.NewOutfitButton = CreateFrame("BUTTON", "Wardrobe_NewOutfitButton", Wardrobe_Outfitframe, "UIPanelButtonTemplate")
Wardrobe.UI.OutfitFrame.NewOutfitButton:SetPoint("TOPLEFT", 10, -10)
Wardrobe.UI.OutfitFrame.NewOutfitButton:SetText('New Set')
Wardrobe.UI.OutfitFrame.NewOutfitButton:SetSize(70, 22)
Wardrobe.UI.OutfitFrame.NewOutfitButton:SetScript("OnClick", function() StaticPopup_Show("Wardrobe_NewOutfit") end)

Wardrobe.UI.OutfitFrame.SaveCurrentlyEquippedButton = CreateFrame("BUTTON", "Wardrobe_SaveCurrentlyEquippedButton", Wardrobe_Outfitframe, "UIPanelButtonTemplate")
Wardrobe.UI.OutfitFrame.SaveCurrentlyEquippedButton:SetPoint("TOPLEFT", 85, -10)
Wardrobe.UI.OutfitFrame.SaveCurrentlyEquippedButton:SetText('Save')
Wardrobe.UI.OutfitFrame.SaveCurrentlyEquippedButton:SetSize(70, 22)
Wardrobe.UI.OutfitFrame.SaveCurrentlyEquippedButton:SetScript("OnClick", function() Wardrobe.SaveCurrentOutfit() end)

Wardrobe.UI.OutfitFrame.DeleteCurrentSet = CreateFrame("BUTTON", "Wardrobe_DeleteCurrentSet", Wardrobe_Outfitframe, "UIPanelButtonTemplate")
Wardrobe.UI.OutfitFrame.DeleteCurrentSet:SetPoint("TOPLEFT", 160, -10)
Wardrobe.UI.OutfitFrame.DeleteCurrentSet:SetText('Delete')
Wardrobe.UI.OutfitFrame.DeleteCurrentSet:SetSize(70, 22)
Wardrobe.UI.OutfitFrame.DeleteCurrentSet:SetScript("OnClick", function() Wardrobe.DeleteCurrentSet() end)

Wardrobe.UI.OutfitFrame.EquipCurrentSet = CreateFrame("BUTTON", "Wardrobe_EquipCurrentSet", Wardrobe_Outfitframe, "UIPanelButtonTemplate")
Wardrobe.UI.OutfitFrame.EquipCurrentSet:SetPoint("TOPLEFT", 160, -40)
Wardrobe.UI.OutfitFrame.EquipCurrentSet:SetText('Equip')
Wardrobe.UI.OutfitFrame.EquipCurrentSet:SetSize(70, 22)
Wardrobe.UI.OutfitFrame.EquipCurrentSet:SetScript("OnClick", function() Wardrobe.IterateSet(Wardrobe.Vars.SelectedOutfit) end)

Wardrobe.UI.OutfitFrame.OutfitDropDown = CreateFrame("FRAME", "GHC_PlayerRoleDropdown", Wardrobe_Outfitframe, "UIDropDownMenuTemplate")
Wardrobe.UI.OutfitFrame.OutfitDropDown:SetPoint("TOPLEFT", -5, -40)
Wardrobe.UI.OutfitFrame.OutfitDropDown.displayMode = nil --"MENU"
UIDropDownMenu_SetWidth(Wardrobe.UI.OutfitFrame.OutfitDropDown, 120)
UIDropDownMenu_SetText(Wardrobe.UI.OutfitFrame.OutfitDropDown, 'Outfits')
function Wardrobe.UI.OutfitFrame.OutfitDropDown_Init()
	UIDropDownMenu_Initialize(Wardrobe.UI.OutfitFrame.OutfitDropDown, function(self, level, menuList)
		local info = UIDropDownMenu_CreateInfo()
        for k, outfit in ipairs(WardrobeDb[UnitGUID('player')].Outfits) do
            info.text = outfit.Name
            info.arg1 = nil
            info.arg2 = nil						
            info.func = function()
                Wardrobe.Vars.SelectedOutfit = outfit.Name
                UIDropDownMenu_SetText(Wardrobe.UI.OutfitFrame.OutfitDropDown, outfit.Name)
                --Wardrobe.EquipCurrentSet()
            end
            info.isNotRadio = true
            --info.hasArrow = true
            --info.menuList = class.MenuList
            UIDropDownMenu_AddButton(info)
        end
	end)
end

function Wardrobe.GetSetItemCount(outfit)
    local c = 0
    if WardrobeDb[UnitGUID('player')] ~= nil then
        for k, outfit in ipairs(WardrobeDb[UnitGUID('player')].Outfits) do
            if outfit.Name == outfit then
                for slot, item in pairs(outfit) do
                    if slot ~= 'Name' then
                        if item ~= 'empty' then
                            c = c + 1
                        end
                    end
                end
            end
        end
    end
    return c
end

function Wardrobe.GetEmptyBagSlot()
	for bag = 1, 4 do
        for slot = 1,GetContainerNumSlots(bag) do
            local item = GetContainerItemID(bag, slot)
            if not item then
                return bag
            end
        end
    end
end

function Wardrobe.GetEmptyBackpackSlot()
	local bag = 0
    for slot = 1, GetContainerNumSlots(bag) do
        local item = GetContainerItemID(bag, slot)
        if not item then
            return bag
        end
    end
end

function Wardrobe.IterateSet(set)
    Wardrobe.DebugMessage('iterate set '..set)
    if WardrobeDb[UnitGUID('player')] ~= nil then
        for k, outfit in ipairs(WardrobeDb[UnitGUID('player')].Outfits) do
            if outfit.Name == set then
                if CursorHasItem() or CursorHasMoney() or CursorHasSpell() then ClearCursor() end
                for slot, itemID in pairs(outfit) do
                    if slot ~= 'Name' then
                        local slotId, textureName = GetInventorySlotInfo(slot)
                        local itemLink = select(2, GetItemInfo(itemID))
                        if itemID == 'empty' then                        
                            if Wardrobe.GetEmptyBagSlot() ~= nil then
                                PickupInventoryItem(slotId)
                                local bagID = Wardrobe.GetEmptyBagSlot() + 19
                                local bagLink = GetInventoryItemLink('player', bagID)
                                PutItemInBag(bagID)
                                Wardrobe.DebugMessage(tostring(outfit.Name..':'..slot..'=empty, moving item into bag '..bagLink))
                            elseif Wardrobe.GetEmptyBackpackSlot() ~= nil then
                                PickupInventoryItem(slotId)
                                PutItemInBackpack()
                                Wardrobe.DebugMessage(tostring(outfit.Name..':'..slot..'=empty, moving item into bag backpack'))
                            else
                                Wardrobe.DebugMessage('no available bag space for items')
                            end
                        else
                            local equipped = false
                            for b = 0, 4 do
                                if equipped == true then break end
                                for s = 1, GetContainerNumSlots(b) do
                                    if equipped == true then break end
                                    local id = GetContainerItemID(b, s)
                                    if id == itemID then
                                        local bagLink = GetInventoryItemLink('player', (b + 19))
                                        if bagLink == nil then bagLink = 'backpack' end --assuming this is the case NOT tested for bank bags
                                        if Wardrobe.Vars.BankOpen == true then
                                            Wardrobe.EquipItemByID(itemID)
                                        else
                                            EquipItemByName(itemID, slotId)
                                        end
                                        --PickupContainerItem(b, s)
                                        --EquipCursorItem(slotId)
                                        --EquipItemByName(itemID, slotId)
                                        equipped = true
                                        Wardrobe.DebugMessage('found '..itemLink..' in bag '..bagLink..' equipping in slot '..slot)                                        
                                    end
                                end
                            end

                        end

                    end
                end
                Wardrobe.Vars.CurrentlyEquippedOutfit = set
            end
        end
    end
end

--this is now an old function, remove in future
function Wardrobe.EquipCurrentSet()
    if WardrobeDb[UnitGUID('player')] ~= nil then
        for k, outfit in ipairs(WardrobeDb[UnitGUID('player')].Outfits) do
            if outfit.Name == Wardrobe.Vars.SelectedOutfit then
                for slot, item in pairs(outfit) do
                    if slot ~= 'Name' then
                        local slotId, textureName = GetInventorySlotInfo(slot)
                        if slotId then
                            if Wardrobe.Vars.BankOpen == true then
                                Wardrobe.EquipItemByID(item)
                            else
                                EquipItemByName(item, slotId)
                            end
                        end
                        if item == 'empty' then
                            local unEquipped = false
                            if CursorHasItem() or CursorHasMoney() or CursorHasSpell() then ClearCursor() end
                            PickupInventoryItem(slotId)
                            for s = 1, 16 do
                                local link = GetContainerItemLink(0, s)
                                if not link then
                                    PutItemInBackpack()
                                    unEquipped = true
                                end
                            end
                            if unEquipped == false then
                                for b = 1, 4 do
                                    for s = 1, GetContainerNumSlots(b) do
                                        local link = GetContainerItemLink(b, s)
                                        if not link then
                                            PutItemInBag(b)
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
end

function Wardrobe.EquipItemByID(id)
	for bag = BANK_CONTAINER, NUM_BAG_SLOTS + NUM_BANKBAGSLOTS do
		for slot = 1,GetContainerNumSlots(bag) do
            local item = GetContainerItemID(bag, slot)
            if item and item == id then
				if CursorHasItem() or CursorHasMoney() or CursorHasSpell() then ClearCursor() end
				PickupContainerItem(bag, slot)
				AutoEquipCursorItem()
				return true
			end
		end
	end
end

function Wardrobe.LoadCharacter()
    if WardrobeDb[UnitGUID('player')] == nil then
        WardrobeDb[UnitGUID('player')] = { Outfits = {}, }
    else
        Wardrobe.FixDb()
    end
    Wardrobe.UI.OutfitFrame.OutfitDropDown_Init()
end

function Wardrobe.CreateNewOutfit(outfitName)
    if WardrobeDb[UnitGUID('player')] ~= nil then
        local add = true
        for k, outfit in ipairs(WardrobeDb[UnitGUID('player')].Outfits) do
            if outfit.Name == outfitName then
                add = false
                Wardrobe.PrintMessage('outfit already exists, choose a new name!')
            end
        end
        if add == true then
            table.insert(WardrobeDb[UnitGUID('player')].Outfits, { Name = outfitName, } )
            Wardrobe.Vars.SelectedOutfit = outfitName
            UIDropDownMenu_SetText(Wardrobe.UI.OutfitFrame.OutfitDropDown, outfitName)
            Wardrobe.PrintMessage('created '..outfitName)
        end
    end
end

function Wardrobe.DeleteCurrentSet()
    if WardrobeDb[UnitGUID('player')] ~= nil then
        for k, outfit in ipairs(WardrobeDb[UnitGUID('player')].Outfits) do
            if outfit.Name == Wardrobe.Vars.SelectedOutfit then
                Wardrobe.PrintMessage('deleted '..outfit.Name)
                table.remove(WardrobeDb[UnitGUID('player')].Outfits, k) --change this as its a terrible idea - create a better remove function
                UIDropDownMenu_SetText(Wardrobe.UI.OutfitFrame.OutfitDropDown, '')
            end
        end
    end
end

function Wardrobe.SaveCurrentOutfit()
    if WardrobeDb[UnitGUID('player')] ~= nil then
        for k, outfit in ipairs(WardrobeDb[UnitGUID('player')].Outfits) do
            if outfit.Name == Wardrobe.Vars.SelectedOutfit then
                for i = 1, 19 do
                    local itemId = GetInventoryItemID("player", i)
                    if itemId then
                        outfit[Wardrobe.DB.InvSlots[i]] = itemId
                    else
                        outfit[Wardrobe.DB.InvSlots[i]] = 'empty'
                    end
                end
                Wardrobe.PrintMessage('saved current items to '..outfit.Name)
            end
        end
    end
end

--first version used INVSLOT_HEAD rather than HEADSLOT so this should update any old databases, can be removed in future
function Wardrobe.FixDb()
    if WardrobeDb[UnitGUID('player')] ~= nil then
        for k, outfit in ipairs(WardrobeDb[UnitGUID('player')].Outfits) do
            for slot, itemId in pairs(outfit) do
                local newSlot = nil
                if string.find(slot, 'INVSLOT') then
                    if string.find(slot, '1') then
                        local oneZero = tostring(string.sub(slot, 1, -2)..'0')
                        newSlot = tostring(string.sub(oneZero, 9)..'SLOT')
                    elseif string.find(slot, '2') then
                        local twoOne = tostring(string.sub(slot, 1, -2)..'1')
                        newSlot = tostring(string.sub(twoOne, 9)..'SLOT')
                    elseif string.find(slot, 'OFFHAND') then
                        newSlot = 'SECONDARYHANDSLOT'
                    elseif string.find(slot, '_HAND') then
                        newSlot = 'HANDSSLOT'
                    elseif string.find(slot, 'BODY') then
                        newSlot = 'SHIRTSLOT'
                    else
                        newSlot = tostring(string.sub(slot, 9)..'SLOT')
                    end
                    outfit[newSlot] = itemId
                    outfit[slot] = nil
                end
            end
        end
    end
end

function Wardrobe.OnEvent(self, event, ...)

    if event == "ADDON_LOADED" and select(1, ...) == 'Wardrobe' then
        if WardrobeDb == nil then
            WardrobeDb = {}
        end
        C_Timer.After(1, function() Wardrobe.LoadCharacter() end)
    end

    if event == 'BANKFRAME_OPENED' then
        Wardrobe.Vars.BankOpen = true
    end
    if event == 'BANKFRAME_CLOSED' then
        Wardrobe.Vars.BankOpen = false
    end

end

Wardrobe.EventFrame:SetScript("OnEvent", Wardrobe.OnEvent)
